# encoding: utf-8


class JustificationBodyDto(object):
    def __init__(
        self,
        order_id,                       # type: str
        operator,                       # type: str
        authorized_by,                  # type: str
        discount_description,           # type: str
        justification,                  # type: str
    ):
        # type: (...) -> None

        self.order_id = order_id
        self.operator = operator
        self.authorized_by = authorized_by
        self.discount_description = discount_description
        self.justification = justification
