import iso8601

from deliveryreport.model import DeliveryOrder


class DeliveryHeaderDto(object):
    def __init__(
            self,
            order   # type: DeliveryOrder
    ):
        # type: (...) -> None

        self.store_name = order.store_name
        self.order_number = order.order_number
        self.order_date = iso8601.parse_date(order.order_date)
        self.partner = order.partner
