from abc import ABCMeta, abstractmethod
from typing import Any


class CouponServiceModel(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_prices(self, posid):
        # type: (int) -> Any

        raise NotImplementedError

