from helper import ExtendedEnum
from messagebus import (
    TokenCreator,
    TokenPriority,
)

MSGGRP_DEVICE_MANAGER = "D"


class ListenedTokens(ExtendedEnum):
    # Processors
    TK_GET_USED_SERVICE = TokenCreator.create_token(TokenPriority.low, MSGGRP_DEVICE_MANAGER, "1")
