from abc import (
    ABCMeta,
    abstractmethod
)
from typing import Any


class ProductInfoServiceModel(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_product_info(
        self,
        language    # type: str
    ):
        # type: (...) -> Any

        raise NotImplementedError
