show_welcome_message = {}
show_final_message = {}

class LoyaltyInteractor(object):
    def __init__(
        self,
        pos_id,      # type: str
    ):
        # type: (...) -> None

        self.pos_id = pos_id

    def set_show_customer_welcome_message(
        self,
        show,    # type: bool
    ):
        # type: (...) -> None

        global show_welcome_message

        if self.pos_id not in show_welcome_message:
            show_welcome_message[self.pos_id] = show

    def get_show_customer_welcome_message(
        self,
    ):
        # type: (...) -> bool

        global show_welcome_message

        if self.pos_id in show_welcome_message:
            return show_welcome_message[self.pos_id]

        return False

    def set_show_customer_final_message(
        self,
        show,    # type: bool
    ):
        # type: (...) -> None

        global show_final_message

        if self.pos_id not in show_final_message:
            show_final_message[self.pos_id] = show

    def get_show_customer_final_message(
        self,
    ):
        # type: (...) -> bool

        global show_final_message

        if self.pos_id in show_final_message:
            return show_final_message[self.pos_id]

        return False
