# -*- coding: utf-8 -*-
class GLAccount(object):
    def __init__(self, manager="", envelope="", justification=""):
        self.manager = manager
        self.envelope = envelope
        self.justification = justification
