from typing import Optional


class Configs(object):

    def __init__(
        self,
        loader_path=None,           # type: Optional[str]
        component_name=None,        # type: Optional[str]
        required_services=None,     # type: Optional[str]
        instances=None,             # type: Optional[int]
        databases_dir=None,         # type: Optional[str]
        custom_data=None,           # type: Optional[str]
        sector_config=None,         # type: dict
    ):
        # type: (...) -> None

        self.loader_path = loader_path
        self.component_name = component_name
        self.databases_dir = databases_dir
        self.instances = instances
        self.required_services = required_services
        self.custom_data = custom_data
        self.sector_config = sector_config
