from typing import List


class SearchCustomerInfoResponse(object):
    def __init__(
            self,
            customer_info,  # type: CustomerInfo
            benefits,       # type: List[Benefit]
     ):
        # type: (...) -> None

        self.customer_info = customer_info

        if benefits:
            self.benefits = benefits
